package xyz.lychee.lagfixer.libs.adventure.chat;

import java.time.Instant;
import java.util.stream.Stream;
import xyz.lychee.lagfixer.libs.adventure.identity.Identified;
import xyz.lychee.lagfixer.libs.adventure.identity.Identity;
import xyz.lychee.lagfixer.libs.adventure.text.Component;
import xyz.lychee.lagfixer.libs.adventure.text.ComponentLike;
import xyz.lychee.lagfixer.libs.annotations.Contract;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.libs.annotations.ApiStatus.NonExtendable;
import xyz.lychee.lagfixer.libs.examination.Examinable;
import xyz.lychee.lagfixer.libs.examination.ExaminableProperty;

@NonExtendable
public interface SignedMessage extends Identified, Examinable {
   @Contract(
      value = "_ -> new",
      pure = true
   )
   @NotNull
   static @NotNull Signature signature(final byte[] signature) {
      return new SignedMessageImpl.SignatureImpl(signature);
   }

   @Contract(
      value = "_, _ -> new",
      pure = true
   )
   static @NotNull SignedMessage system(final @NotNull String message, final @Nullable ComponentLike unsignedContent) {
      return new SignedMessageImpl(message, ComponentLike.unbox(unsignedContent));
   }

   @Contract(
      pure = true
   )
   @NotNull Instant timestamp();

   @Contract(
      pure = true
   )
   long salt();

   @Contract(
      pure = true
   )
   @Nullable
   @Nullable Signature signature();

   @Contract(
      pure = true
   )
   @Nullable Component unsignedContent();

   @Contract(
      pure = true
   )
   @NotNull String message();

   @Contract(
      pure = true
   )
   default boolean isSystem() {
      return this.identity() == Identity.nil();
   }

   @Contract(
      pure = true
   )
   default boolean canDelete() {
      return this.signature() != null;
   }

   default @NotNull Stream<? extends ExaminableProperty> examinableProperties() {
      return Stream.of(ExaminableProperty.of("timestamp", this.timestamp()), ExaminableProperty.of("salt", this.salt()), ExaminableProperty.of("signature", this.signature()), ExaminableProperty.of("unsignedContent", this.unsignedContent()), ExaminableProperty.of("message", this.message()));
   }

   @NonExtendable
   public interface Signature extends Examinable {
      @Contract(
         pure = true
      )
      byte[] bytes();

      default @NotNull Stream<? extends ExaminableProperty> examinableProperties() {
         return Stream.of(ExaminableProperty.of("bytes", this.bytes()));
      }
   }
}
